(define-module (ordo inventory)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module ((ordo connection) #:select (local-connection))
  #:export (host
            host?
            host-name
            host-connection
            host-tags
            add-host!
            resolve-hosts))

(define-record-type <host>
  (make-host name connection tags)
  host?
  (name host-name)
  (connection host-connection)
  (tags host-tags))

(define (host name connection . tags)
  (make-host name connection tags))

(define (tagged-every? wanted-tags)
  (lambda (h)
    (lset= equal? wanted-tags (lset-intersection equal? (host-tags h) wanted-tags))))

(define (tagged-any? wanted-tags)
  (lambda (h)
    (not (null? (lset-intersection equal? (host-tags h) wanted-tags)))))

(define (named? hostname)
  (lambda (h)
    (string=? (host-name h) hostname)))

(define (resolve-hosts inventory expr)
  (match expr
    ("localhost" (list (or (find (named? "localhost") inventory)
                           (make-host "localhost" (local-connection) '()))))
    ((? string? hostname) (filter (named? hostname) inventory))
    ('all inventory)
    (('tagged tag) (filter (tagged-every? (list tag)) inventory))
    (('tagged/every tag . tags) (filter (tagged-every? (cons tag tags)) inventory))
    (('tagged/any tag . tags) (filter (tagged-any? (cons tag tags)) inventory))))
